#include "Explosion.h"
#include "Camera.h"
#include "sprite.h"

#include "Utils.h"

void ExplosionUpdate(ObjectStruct *explosionObject);
void ExplosionDestroy(ObjectStruct *explosionObject);
void ExplosionDraw(ObjectStruct *explosionObject);

ObjectStruct *ExplosionCreate(fix32 x, fix32 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_EFFECT,
                                                      ExplosionUpdate,
                                                      ExplosionDestroy,
                                                      ExplosionDraw,
                                                      CollisionFunctionStub);

    if (!object)
    {
        KDebug_Alert("Explosion could not be created!");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 1;

    object->sprite = SPR_addSprite(&explosion6, fix32ToInt(object->x) - fix32ToInt(camX), fix32ToInt(object->y) - fix32ToInt(camY), TILE_ATTR(PAL3, TRUE, FALSE, FALSE));
    SPR_setAnim(object->sprite, 0);
    SPR_setFrame(object->sprite, 0);

    return object;
}

void ExplosionUpdate(ObjectStruct *object)
{
    if(object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }
    
    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > 3)
    {
        object->animationFrame++;
        if (object->animationFrame > 7)
        {
            object->objectState = OBJECTSTATE_INACTIVE;
            
            return;
        }
        else
        {
            SPR_setFrame(object->sprite, object->animationFrame);
        }
        object->animationSubFrameCounter = 0;
    }

    SPR_setPosition(object->sprite, fix32ToInt(object->x - camX), fix32ToInt(object->y - camY));
}

void ExplosionDestroy(ObjectStruct *object)
{
    SPR_releaseSprite(object->sprite);
}

void ExplosionDraw(ObjectStruct *explosionObject)
{
}
