#include <genesis.h>
#include "Camera.h"
#include "Hellhound.h"
#include "Utils.h"
#include "JoypadManager.h"
#include "Constants.h"
#include <sprite.h>

#define HELLHOUND_ANIM_RUN 0
#define HELLHOUND_ANIM_IDLE 1
#define HELLHOUND_ANIM_JUMP 2

void HellhoundUpdate(ObjectStruct *object);
void HellhoundDestroy(ObjectStruct *object);
void HellhoundDraw(ObjectStruct *object);
void HellhoundCollide(ObjectStruct *object, ObjectStruct *collidedObject);

char buffer[256];
ObjectStruct *Hellhound = NULL;
u8 first = 1;

ObjectStruct *HellhoundCreate(fix32 x, fix32 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      HellhoundUpdate,
                                                      HellhoundDestroy,
                                                      HellhoundDraw,
                                                      HellhoundCollide);
    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("enemy could not be created!");
#endif
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->startSpeedX = FIX32(-3);
    object->speedX = 0;
    object->speedY = 0;

    object->rectLeft = 8;
    object->rectRight = 32;
    object->rectTop = 24;
    object->rectBottom = 48;

    object->collisionRectLeft = object->rectLeft;
    object->collisionRectRight = object->rectRight;
    object->collisionRectTop = object->rectTop;
    object->collisionRectBottom = object->rectBottom;

    object->goingRight = 0;
    object->healthPoints = 1;
    object->damage = 1;
    object->tileBottomCheckX = 6;
    object->tileBottomCheckY = 6;

    object->tileSideCheckX = 5;
    object->tileSideCheckY = 1;

    object->simpleCollision = 1;

    object->animationSubFrameCounter = 0;
    object->animationFrame = 0;

    object->idling = 0;

    object->invulnerableTimer = 0;

    object->isAlive = 1;

    Hellhound = object;

#ifdef DEBUG_MODE
    object->collisionSprite[0] = SPR_addSprite(&red_tl, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    object->collisionSprite[1] = SPR_addSprite(&red_tr, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    object->collisionSprite[2] = SPR_addSprite(&red_bl, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    object->collisionSprite[3] = SPR_addSprite(&red_br, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
#endif

    KLog("hellhound created");

    return object;
}

void HellhoundUpdate(ObjectStruct *object)
{
    if (!object->isAlive)
    {
        return;
    }

    u8 oldGrounded = object->grounded;

    CheckBottomCollision(object);
    CheckSideCollision(object);

    // jump if lose ground
    if (!object->grounded && oldGrounded)
    {
        Jump(object, FIX32(-4));
        object->animationFrame = 0;
        object->animationSubFrameCounter = 0;

        object->collisionRectLeft = 8;
        object->collisionRectRight = 40;
        object->collisionRectTop = 8;
        object->collisionRectBottom = 32;
    }
    else if (object->grounded && !oldGrounded)
    {
        if (object->speedX == 0)
        {
            object->animationFrame = 0;
            object->animationSubFrameCounter = 0;
            if (!first)
            {
                object->looping = 1;
            }
            object->idling = 1;
            first = 0;
        }
        object->collisionRectLeft = object->rectLeft;
        object->collisionRectRight = object->rectRight;
        object->collisionRectTop = object->rectTop;
        object->collisionRectBottom = object->rectBottom;
    }

    //object->x += object->speedX;
    object->y += object->speedY;

    if (object->wasVisible && !object->visible)
    {
        KLog("releasing sprite");
        SPR_releaseSprite(object->sprite);
    }
    else if (!object->wasVisible && object->visible)
    {
        KLog("adding sprite");
        object->sprite = SPR_addSprite(&hellhound, fix32ToInt(object->x) - fix32ToInt(camX), fix32ToInt(object->y) - fix32ToInt(camY), TILE_ATTR(PAL3, TRUE, FALSE, FALSE));
    }
    object->wasVisible = object->visible;

    if (!object->visible)
    {
        return;
    }

    if (object->idling)
    {
        SPR_setAnim(object->sprite, HELLHOUND_ANIM_IDLE);
    }
    else if (!object->grounded)
    {
        SPR_setAnim(object->sprite, HELLHOUND_ANIM_JUMP);
    }
    else if (object->speedX != 0)
    {
        SPR_setAnim(object->sprite, HELLHOUND_ANIM_RUN);
    }

    SPR_setPosition(object->sprite, fix32ToInt(object->x - camX), fix32ToInt(object->y - camY));

#ifdef DEBUG_MODE
    SPR_setPosition(object->collisionSprite[0], fix32ToInt(object->x - camX) + object->collisionRectLeft, fix32ToInt(object->y - camY) + object->collisionRectTop);
    SPR_setPosition(object->collisionSprite[1], fix32ToInt(object->x - camX) + object->collisionRectRight - 8, fix32ToInt(object->y - camY) + object->collisionRectTop);
    SPR_setPosition(object->collisionSprite[2], fix32ToInt(object->x - camX) + object->collisionRectLeft, fix32ToInt(object->y - camY) + object->collisionRectBottom - 8);
    SPR_setPosition(object->collisionSprite[3], fix32ToInt(object->x - camX) + object->collisionRectRight - 8, fix32ToInt(object->y - camY) + object->collisionRectBottom - 8);
#endif

    if (!object->grounded)
    {
        if (object->animationSubFrameCounter > 3)
        {
            object->animationSubFrameCounter = 0;
            object->animationFrame++;
        }

        if (object->speedY < FIX32(0))
        {
            if (object->animationFrame > 3)
            {
                object->animationFrame = 3;
            }
        }
        else
        {
            object->animationFrame = 4;
        }
    }
    else if (object->idling)
    {
        if (object->animationSubFrameCounter > 5)
        {
            object->animationSubFrameCounter = 0;
            if (object->looping)
            {
                if (object->animationFrame == 0)
                {
                    object->speedX = object->startSpeedX;
                    object->looping = 0;
                    object->idling = 0;
                }
                else
                {
                    object->animationFrame--;
                }
            }
            else
            {
                object->animationFrame++;
                if (object->animationFrame > 5)
                {
                    object->animationFrame = 5;
                    object->looping = 1;
                }
            }
        }
    }
    else
    {
        if (object->animationSubFrameCounter > 3)
        {
            object->animationSubFrameCounter = 0;
            object->animationFrame++;
        }

        if (object->animationFrame > 4)
        {
            object->animationFrame = 0;
        }
    }

    if (object->invulnerableTimer > 0)
    {
        object->invulnerableTimer--;
        if (object->invulnerableTimer == 0)
        {
            SPR_setPalette(object->sprite, PAL3);
        }
    }

    object->animationSubFrameCounter++;
}

void HellhoundDestroy(ObjectStruct *object)
{
    KLog("hellhound destroy");
    SPR_releaseSprite(object->sprite);
}

void HellhoundDraw(ObjectStruct *object)
{
    if (!object->isAlive)
    {
        return;
    }

    if (!object->visible)
    {
        return;
    }

    SPR_setFrame(object->sprite, object->animationFrame);
}

void HellhoundCollide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    if (object->invulnerableTimer > 0)
    {
        return;
    }

    if (!object->isAlive)
    {
        return;
    }

    if (!object->visible)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    object->invulnerableTimer = 10;
    SPR_setPalette(object->sprite, PAL2);

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x, object->y);
        object->isAlive = 0;
        object->objectState = OBJECTSTATE_INACTIVE;
    }
}