#include "LevelManager.h"
#include "Palette.h"
#include "Player.h"
#include <genesis.h>
#include "ObjectSpawner.h"
#include "gfx.h"
#include "sprite.h"

typedef struct Level
{
    Image *planea;
    Image *planeb;
} Level;

enum LEVEL_STATE
{
    LOADING,
    UPDATING
};

u8 currentLevelIndex = 0;
u16 baseTile;
u16 maxCollisionTile;
Level *currentLevel;

Map *currentMap;

enum LEVEL_STATE state;

Level level1 =
{
    &level1a,
    &level1b
};

Level* levels[1] =
{
    &level1
};


void LevelManagerInit()
{
    state = LOADING;
    currentLevel = levels[0];
    PlayerSetup();
}

void LevelManagerUpdate()
{
    switch (state)
    {
    case LOADING:
        ObjectSpawner_Init();
        SetPaletteAllBlack();
        currentMap = unpackMap(currentLevel->planea->map, NULL);

        baseTile = currentMap->tilemap[currentMap->w << 1];
        maxCollisionTile = 0;
        for (int i = 0; i < currentMap->w; i++)
        {
            if (currentMap->tilemap[i] == baseTile)
            {
                break;
            }
            if (currentMap->tilemap[i] > maxCollisionTile)
            {
                maxCollisionTile = currentMap->tilemap[i];
            }
            if (currentMap->tilemap[i + currentMap->w] > maxCollisionTile)
            {
                maxCollisionTile = currentMap->tilemap[i + currentMap->w];
            }
            currentMap->tilemap[i] = baseTile;
            currentMap->tilemap[i + currentMap->w] = baseTile;
        }

        u16 ind = TILE_USERINDEX;
        VDP_loadTileSet(currentLevel->planea->tileset, ind, 0);
        VDP_setMapEx(PLAN_A, currentMap, TILE_ATTR_FULL(PAL0, FALSE, FALSE, FALSE, ind), 0, 0, 0, 0, planWidth, planHeight);
        ind += currentLevel->planea->tileset->numTile;

        VDP_drawImageEx(PLAN_B, currentLevel->planeb, TILE_ATTR_FULL(PAL1, FALSE, FALSE, FALSE, ind), 0, 0, FALSE, TRUE);

        SetPalette(PAL0, currentLevel->planea->palette->data);
        SetPalette(PAL1, currentLevel->planeb->palette->data);
        SetPalette(PAL2, ninja_top_sprite.palette->data);
        SetPalette(PAL3, hellhound.palette->data);
        FadeIn();
        state = UPDATING;
        break;
    case UPDATING:
        ObjectManager_UpdateObjects();
        ObjectSpawner_Update();
        ObjectManager_DrawObjects();
        break;
    }
}