#include "ObjectManager.h"
#include "Camera.h"
#include "Utils.h"
#include "Constants.h"
#include "Player.h"

extern u16 spriteNum;

#define MAX_OBJECTS 60

ObjectStruct objects[MAX_OBJECTS];
u16 numActiveObjects = 0;

ObjectStruct *firstObjectOfType[MAX_OBJECT_TYPES];
ObjectStruct *lastObjectOfType[MAX_OBJECT_TYPES];

u16 (*ActiveAreaIsWithinSpan)(ObjectStruct *object);

void ObjectManager_Init()
{
    numActiveObjects = 0;
}

ObjectStruct *ObjectManager_CreateObject(u16 objectType,
                                         ObjectFunctionType updateFunction,
                                         ObjectFunctionType destroyFunction,
                                         ObjectFunctionType drawFunction,
                                         ObjectCollisionFunctionType collisionFunction)
{
    if (numActiveObjects >= MAX_OBJECTS)
    {
        return NULL;
    }

    ObjectStruct *object = objects + numActiveObjects;

    object->updateFunction = updateFunction;
    object->destroyFunction = destroyFunction;
    object->drawFunction = drawFunction;
    object->collisionFunction = collisionFunction;
    object->objectState = OBJECTSTATE_ACTIVE;
    object->objectType = objectType;

    numActiveObjects++;

    return object;
}

void DrawObjects(u16 objectType)
{
    if (*(firstObjectOfType + objectType) == NULL)
        return;

    ObjectStruct *object = *(firstObjectOfType + objectType);

    while (object != NULL)
    {
        object->drawFunction(object);
        object = (ObjectStruct *)object->nextVisibleObjectOfSameType;
    }
}

void ObjectManager_DrawObjects()
{
    playerObject->drawFunction(playerObject);
    DrawObjects(TYPE_ENEMY);
}

void ObjectManager_UpdateVDP()
{
    VDP_updateSprites(40, TRUE);
}

void ProcessCollisions(u16 firstType, u16 secondType)
{
    // if (*(firstObjectOfType + firstType )== NULL) return;
    // if (*(firstObjectOfType + secondType) == NULL) return;

    // ObjectStruct* objectOfFirstType = *(firstObjectOfType + firstType);
    // ObjectStruct* objectOfSecondType = NULL;

    // while (objectOfFirstType != NULL)
    // {
    //     if (objectOfFirstType->healthPoints > 0 &&
    //         objectOfFirstType->objectState == OBJECTSTATE_ACTIVE)
    //     {
    //         objectOfSecondType = *(firstObjectOfType + secondType);

    //         while (objectOfSecondType != NULL)
    //         {
    //             if (objectOfSecondType->healthPoints > 0 &&
    //                 objectOfSecondType->objectState == OBJECTSTATE_ACTIVE)
    //             {
    //                 if (!(objectOfFirstType->x + objectOfFirstType->rectLeft   > objectOfSecondType->x + objectOfSecondType->rectRight ||
    //                       objectOfFirstType->x + objectOfFirstType->rectRight  < objectOfSecondType->x + objectOfSecondType->rectLeft ||
    //                       objectOfFirstType->y + objectOfFirstType->rectTop    > objectOfSecondType->y + objectOfSecondType->rectBottom ||
    //                       objectOfFirstType->y + objectOfFirstType->rectBottom < objectOfSecondType->y + objectOfSecondType->rectTop))
    //                 {
    //                     objectOfFirstType->collisionFunction(objectOfFirstType, objectOfSecondType);
    //                     objectOfSecondType->collisionFunction(objectOfSecondType, objectOfFirstType);
    //                 }
    //             }

    //             objectOfSecondType = objectOfSecondType->nextVisibleObjectOfSameType;
    //         }
    //     }

    //     objectOfFirstType = objectOfFirstType->nextVisibleObjectOfSameType;
    // }
}

void ProcessCollisionsAgainstPlayer(u16 objectType)
{
    if (*(firstObjectOfType + objectType) == NULL)
        return;

    // no need to do anything if we're dead.
    if (playerObject->objectState != OBJECTSTATE_ACTIVE)
    {
        return;
    }

    ObjectStruct *firstObject = *(firstObjectOfType + objectType);

    while (firstObject != NULL)
    {
        if (firstObject->healthPoints > 0 &&
            firstObject->objectState == OBJECTSTATE_ACTIVE)
        {
            if (!(firstObject->x + firstObject->rectLeft > playerObject->x + playerObject->rectRight ||
                  firstObject->x + firstObject->rectRight < playerObject->x + playerObject->rectLeft ||
                  firstObject->y + firstObject->rectTop > playerObject->y + playerObject->rectBottom ||
                  firstObject->y + firstObject->rectBottom < playerObject->y + playerObject->rectTop))
            {
                firstObject->collisionFunction(firstObject, playerObject);
                playerObject->collisionFunction(playerObject, firstObject);
            }
        }

        firstObject = firstObject->nextVisibleObjectOfSameType;
    }
}

void ProcessPlayerMelee(u16 objectType)
{
    if (*(firstObjectOfType + objectType) == NULL)
        return;

    // no need to do anything if we're dead.
    if (playerObject->objectState != OBJECTSTATE_ACTIVE)
    {
        return;
    }

    ObjectStruct *firstObject = *(firstObjectOfType + objectType);

    while (firstObject != NULL)
    {
        if (firstObject->healthPoints > 0 &&
            firstObject->objectState == OBJECTSTATE_ACTIVE)
        {
            if (isMeleeOverlapping(firstObject))
            {
                firstObject->collisionFunction(firstObject, playerObject);
                playerObject->collisionFunction(playerObject, firstObject);
            }
        }

        firstObject = firstObject->nextVisibleObjectOfSameType;
    }
}

void ProcessAllCollisions()
{
    //ProcessCollisions(TYPE_PLAYER_PROJECTILE, TYPE_ENEMY);
    if (playerObject->attacking)
    {
        ProcessPlayerMelee(TYPE_ENEMY);
    }
}

void UpdateObjects()
{
    ObjectStruct *tempObject = objects;
    ObjectStruct *lastObject = (objects + numActiveObjects);

    memset(firstObjectOfType, NULL, sizeof(firstObjectOfType));
    memset(lastObjectOfType, NULL, sizeof(lastObjectOfType));

    ObjectStruct **firstObjectOfTypePointer = firstObjectOfType;
    ObjectStruct **lastObjectOfTypePointer = lastObjectOfType;

    while (tempObject != lastObject)
    {
        tempObject->updateFunction(tempObject);
        tempObject->visible = isVisible(fix32ToInt(camX) - tempObject->rectRight, fix32ToInt(camX) + SCREEN_WIDTH, fix32ToInt(tempObject->x));

        if (tempObject->objectState == OBJECTSTATE_INACTIVE)
        {
            tempObject->destroyFunction(tempObject); // what happens if they create any new objects here???

            numActiveObjects--;

            // set the new last object
            lastObject = (objects + numActiveObjects);

            // swap the top of the stack with the current dead object.
            // this will eliminate any holes in the stack.
            if (tempObject != lastObject)
            {
                memcpy(tempObject, lastObject, sizeof(ObjectStruct));
            }

            // we don't move the tempObject pointer because we've copied a new object
            // to where the dead one is. So we want to reprocess it.
        }
        else if (tempObject->objectState == OBJECTSTATE_ACTIVE)
        {
            tempObject->lifetime++; // loops after 65535. Maximum of 18.2 minutes.

            if (tempObject->visible)
            {
                // update ordering of same type.
                u16 objectType = tempObject->objectType;

                if (*(firstObjectOfTypePointer + objectType) == NULL)
                {
                    *(firstObjectOfTypePointer + objectType) = tempObject;
                }

                if (*(lastObjectOfTypePointer + objectType) != NULL)
                {
                    (*(lastObjectOfTypePointer + objectType))->nextVisibleObjectOfSameType = tempObject;
                }

                tempObject->nextVisibleObjectOfSameType = NULL;
                *(lastObjectOfTypePointer + objectType) = tempObject;
            }

            tempObject++;
        }
    }
}

void ObjectManager_UpdateObjects()
{
    playerObject->updateFunction(playerObject);
    playerObject->lifetime++;

    UpdateCamera();

    UpdateObjects();

    ProcessAllCollisions();
}