#ifndef OBJECTSTRUCT_INCLUDE_H
#define OBJECTSTRUCT_INCLUDE_H

#include <genesis.h>

#define OBJECTSTATE_ACTIVE 0
#define OBJECTSTATE_INACTIVE 1

// object types.
#define TYPE_PLAYER_PROJECTILE 0
#define TYPE_EFFECT 1
#define TYPE_ENEMY 2
#define TYPE_ENEMY_PROJECTILE 3
#define TYPE_PICKUP 4
#define TYPE_PLATFORM 5
#define MAX_OBJECT_TYPES 6
// specially treated object types.
#define TYPE_NONE 6
#define TYPE_PLAYER 7

#define MAX_SPRITES 3

typedef struct object_struct
{
    s16 objectStreamerId;

    struct object_struct *nextVisibleObjectOfSameType;

    void (*updateFunction)(struct object_struct *objectStruct);
    void (*destroyFunction)(struct object_struct *objectStruct);
    void (*drawFunction)(struct object_struct *objectStruct);
    void (*collisionFunction)(struct object_struct *objectStruct, struct object_struct *collidedObject);

    fix32 x;
    fix32 y;
    fix32 startSpeedX;
    fix32 speedX;
    fix32 speedY;

    u8 objectType;
    u8 visible;
    u16 lifetime; // one tick every frame. 60 ticks every second. 3600 ticks every minute. Max 18.2 minutes. only for temporary objects.

    u16 objectState;

    s16 rectLeft;
    s16 rectRight;
    s16 rectTop;
    s16 rectBottom;

    s16 collisionRectLeft;
    s16 collisionRectRight;
    s16 collisionRectTop;
    s16 collisionRectBottom;

    s16 attackRectLeft;
    s16 attackRectRight;
    s16 attackRectTop;
    s16 attackRectBottom;

    u8 healthPoints;
    u8 damage;

    u8 goingRight;

    u8 grounded;
    u8 attacking;
    u8 jumping;
    u8 doubleJumping;
    u8 idling;
    u8 crouching;

    u8 isAlive;

    u8 tileBottomCheckX;
    u8 tileBottomCheckY;

    u8 tileSideCheckX;
    u8 tileSideCheckY;

    u8 simpleCollision;

    u16 animationFrame;
    u16 animationSubFrameCounter;

    u16 invulnerableTimer;

    u8 wasVisible;
    u8 looping;

    Sprite *collisionSprite[4];
    Sprite *sprite;
} object_struct;

typedef struct object_struct ObjectStruct;

typedef ObjectStruct *(*ObjectCreateFunctionType)(fix32 x, fix32 y);
typedef void (*ObjectFunctionType)(ObjectStruct *objectStruct);
typedef void (*ObjectCollisionFunctionType)(ObjectStruct *objectStruct, ObjectStruct *collidedObject);

#endif