#include "Player.h"
#include "JoypadManager.h"
#include "LevelManager.h"
#include "Camera.h"
#include "Utils.h"
#include "Constants.h"

#include "sprite.h"

#include "Hellhound.h"

#include <genesis.h>

#define MAX_LIVES 3
#define MAX_HEALTH_POINTS 4

#define TOP_ANIM_ATTACK 0
#define TOP_ANIM_RUN 1
#define TOP_ANIM_IDLE 1

#define BOTTOM_ANIM_RUN 0
#define BOTTOM_ANIM_JUMP 1
#define BOTTOM_ANIM_CROUCH 2

#define ATTACK_ANIM_ATTACK 0

#define TOP_SPRITE 0
#define BOTTOM_SPRITE 1
#define ATTACK_SPRITE 2

ObjectStruct playerStruct;
ObjectStruct *playerObject;
ObjectStruct *player;

void PlayerUpdate(ObjectStruct *playerObject);
void PlayerDestroy(ObjectStruct *playerObject);
void PlayerDraw(ObjectStruct *playerObject);
void PlayerCollide(ObjectStruct* playerObject, ObjectStruct* collidedObject);

char buffer[256];
Sprite *sprites[3];

u16 attackFrame = 0;
u16 attackSubFrame = 0;

void PlayerSetup()
{
    playerObject = &playerStruct;
    player = playerObject;
    playerObject->updateFunction = PlayerUpdate;
    playerObject->destroyFunction = PlayerDestroy;
    playerObject->drawFunction = PlayerDraw;
    playerObject->collisionFunction = PlayerCollide;
    playerObject->objectState = OBJECTSTATE_ACTIVE;

    playerObject->objectType = TYPE_PLAYER;
    playerObject->damage = 1;
    playerObject->healthPoints = MAX_HEALTH_POINTS;

    playerObject->rectLeft = 4;
    playerObject->rectRight = 20;
    playerObject->rectTop = 0;
    playerObject->rectBottom = 32;

    playerObject->collisionRectLeft = playerObject->rectLeft;
    playerObject->collisionRectRight = playerObject->rectRight;
    playerObject->collisionRectTop = playerObject->rectTop;
    playerObject->collisionRectBottom = playerObject->rectBottom;

    playerObject->x = FIX32(0);
    playerObject->y = FIX32(96);
    playerObject->speedX = 0;
    playerObject->startSpeedX = FIX32(2);
    playerObject->speedY = FIX32(0);
    playerObject->goingRight = 1;

    playerObject->tileBottomCheckX = 3;
    playerObject->tileBottomCheckY = 4;
    playerObject->tileSideCheckX = 2;
    playerObject->tileSideCheckY = 5;

    playerObject->animationFrame = 0;
    playerObject->animationSubFrameCounter = 0;

    playerObject->crouching = 0;
    playerObject->jumping = 0;
    playerObject->attacking = 0;

    sprites[TOP_SPRITE] = SPR_addSprite(&ninja_top_sprite, fix32ToInt(playerObject->x), fix32ToInt(playerObject->y), TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    sprites[BOTTOM_SPRITE] = SPR_addSprite(&ninja_bottom_sprite, fix32ToInt(playerObject->x), fix32ToInt(playerObject->y + FIX32(16.0)), TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    sprites[ATTACK_SPRITE] = SPR_addSprite(&ninja_attack_sprite, fix32ToInt(playerObject->x), fix32ToInt(playerObject->y), TILE_ATTR(PAL2, TRUE, FALSE, FALSE));

    SPR_setAnim(sprites[TOP_SPRITE], TOP_ANIM_IDLE);
    SPR_setAnim(sprites[BOTTOM_SPRITE], 0);
    SPR_setAnim(sprites[ATTACK_SPRITE], 0);
    SPR_setFrame(sprites[ATTACK_SPRITE], 0);

#ifdef DEBUG_MODE
    playerObject->collisionSprite[0] = SPR_addSprite(&red_tl, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    playerObject->collisionSprite[1] = SPR_addSprite(&red_tr, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    playerObject->collisionSprite[2] = SPR_addSprite(&red_bl, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
    playerObject->collisionSprite[3] = SPR_addSprite(&red_br, 0, 0, TILE_ATTR(PAL2, TRUE, FALSE, FALSE));
#endif
}

void HandleInput()
{
    u8 stopCrouching = 0;

    if (joystickState.cPressed)
    {
        //player = Hellhound;
    }

    if (joystickState.bPressed && !player->attacking)
    {
        player->attacking = 1;
        attackFrame = 0;
        attackSubFrame = 0;
        SPR_setAnim(sprites[TOP_SPRITE], TOP_ANIM_ATTACK);
        SPR_setAnim(sprites[ATTACK_SPRITE], ATTACK_ANIM_ATTACK);
    }

    if (joystickState.aPressed)
    {
        if (player->grounded || (player->jumping == 1))
        {
            stopCrouching = 1;
            SPR_setAnim(sprites[BOTTOM_SPRITE], BOTTOM_ANIM_JUMP);
            player->animationFrame = 0;
            player->animationSubFrameCounter = 0;
            Jump(player, FIX32(-5.5));
        }
    }

    fix32 oldSpeedX = player->speedX;
    if (joystate & BUTTON_LEFT)
    {
        stopCrouching = 1;
        player->goingRight = 0;
        player->speedX = -player->startSpeedX;
    }
    else if (joystate & BUTTON_RIGHT)
    {
        stopCrouching = 1;
        player->goingRight = 1;
        player->speedX = player->startSpeedX;
    }
    else if (player->grounded && joystate & BUTTON_DOWN)
    {
        if (!player->crouching)
        {
            player->y += FIX32(7);

            player->collisionRectTop += 3;
            player->collisionRectBottom -= 7;
            player->attackRectTop += 3;
            player->attackRectBottom -= 7;

            player->animationFrame = 0;
            player->animationSubFrameCounter = 0;
            SPR_setAnim(sprites[BOTTOM_SPRITE], BOTTOM_ANIM_CROUCH);
        }
        player->crouching = 1;
        player->speedX = 0;
    }
    else
    {
        stopCrouching = 1;
        player->speedX = 0;
    }

    if (stopCrouching)
    {
        if (player->crouching)
        {
            player->y -= FIX32(7);

            player->collisionRectTop -= 3;
            player->collisionRectBottom += 7;
            player->attackRectTop += 3;
            player->attackRectBottom -= 7;

            player->crouching = 0;
            if (!player->jumping)
            {
                SPR_setAnim(sprites[BOTTOM_SPRITE], BOTTOM_ANIM_RUN);
            }
        }
    }

    if (oldSpeedX != player->speedX)
    {
        player->animationFrame = 0;
        if (player->speedX != 0)
        {
            player->animationFrame = 1;
        }
        player->animationSubFrameCounter = 0;
    }
}

void PlayerUpdate(ObjectStruct *playerObject)
{
    HandleInput();

    u8 wasGrounded = playerObject->grounded;

    if (playerObject->x >= FIX32(currentMap->w * 8))
    {
        playerObject->x = FIX32(currentMap->w * 8);
    }
    else if (playerObject->x <= 0)
    {
        playerObject->x = 0;
    }

    CheckBottomCollision(playerObject);
    CheckTopCollision(playerObject);
    CheckSideCollision(playerObject);

    // landing
    if (playerObject->grounded && !wasGrounded)
    {
        SPR_setAnim(sprites[BOTTOM_SPRITE], BOTTOM_ANIM_RUN);
        player->animationFrame = 0;
        player->animationSubFrameCounter = 0;
    }

    playerObject->x += playerObject->speedX;

    s16 crouchingOffset = 0;
    if (playerObject->crouching)
    {
        crouchingOffset = 3;
    }

    playerObject->attackRectLeft = playerObject->collisionRectLeft;
    playerObject->attackRectRight = playerObject->collisionRectRight;
    playerObject->attackRectTop = playerObject->collisionRectTop;
    playerObject->attackRectBottom = playerObject->collisionRectBottom - 15;
    if (player->crouching)
    {
        playerObject->attackRectBottom += 10;
    }

    if (playerObject->goingRight)
    {
        SPR_setPosition(sprites[ATTACK_SPRITE], fix32ToInt(playerObject->x - camX) + 24, fix32ToInt(playerObject->y - camY) + crouchingOffset);
        SPR_setHFlip(sprites[TOP_SPRITE], TRUE);
        SPR_setHFlip(sprites[BOTTOM_SPRITE], TRUE);
        SPR_setHFlip(sprites[ATTACK_SPRITE], TRUE);

        playerObject->attackRectLeft += 16;
        playerObject->attackRectRight += 30;
    }
    else
    {
        SPR_setPosition(sprites[ATTACK_SPRITE], fix32ToInt(playerObject->x - camX) - 32, fix32ToInt(playerObject->y - camY) + crouchingOffset);
        SPR_setHFlip(sprites[TOP_SPRITE], FALSE);
        SPR_setHFlip(sprites[BOTTOM_SPRITE], FALSE);
        SPR_setHFlip(sprites[ATTACK_SPRITE], FALSE);

        playerObject->attackRectLeft -= 30;
        playerObject->attackRectRight -= 16;
    }

    // set sprites position
    SPR_setPosition(sprites[TOP_SPRITE], fix32ToInt(playerObject->x - camX), fix32ToInt(playerObject->y - camY) + crouchingOffset);
    SPR_setPosition(sprites[BOTTOM_SPRITE], fix32ToInt(playerObject->x - camX), fix32ToInt(playerObject->y - camY) + 16);

#ifdef DEBUG_MODE
    if (!playerObject->attacking)
    {
        DrawCollisionRect(playerObject, playerObject->collisionRectLeft, playerObject->collisionRectTop, playerObject->collisionRectRight, playerObject->collisionRectBottom);
    }
    else
    {
        DrawCollisionRect(playerObject, playerObject->attackRectLeft, playerObject->attackRectTop, playerObject->attackRectRight, playerObject->attackRectBottom);
    }
#endif

    if (playerObject->attacking)
    {
        if (attackSubFrame > 4)
        {
            attackSubFrame = 0;
            attackFrame++;
        }
        if (attackFrame > 1)
        {
            attackFrame = 0;
            playerObject->attacking = 0;
            SPR_setAnim(sprites[TOP_SPRITE], TOP_ANIM_IDLE);
        }
    }

    if (playerObject->jumping)
    {
        if (playerObject->speedY < 0)
        {
            playerObject->animationFrame = 0;
        }
        else
        {
            playerObject->animationFrame = 1;
        }
    }
    else if (playerObject->speedX != 0)
    {
        if (playerObject->animationSubFrameCounter > 7)
        {
            playerObject->animationSubFrameCounter = 0;
            playerObject->animationFrame++;
        }
        if (playerObject->animationFrame > 1)
        {
            playerObject->animationFrame = 0;
        }
    }
    else
    {
        if (playerObject->animationSubFrameCounter > 6)
        {
            playerObject->animationSubFrameCounter = 0;
            playerObject->animationFrame++;
        }
        if (playerObject->animationFrame > 1)
        {
            playerObject->animationFrame = 0;
        }
    }
    playerObject->animationSubFrameCounter++;
    attackSubFrame++;
}

void PlayerDraw(ObjectStruct *playerObject)
{
    if (playerObject->attacking)
    {
        SPR_setFrame(sprites[TOP_SPRITE], attackFrame);
        SPR_setFrame(sprites[ATTACK_SPRITE], attackFrame);
    }
    else
    {
        if (player->crouching)
        {
            SPR_setFrame(sprites[TOP_SPRITE], 0);
        }
        else
        {
            SPR_setFrame(sprites[TOP_SPRITE], playerObject->animationFrame);
        }
        SPR_setFrame(sprites[ATTACK_SPRITE], 0);
    }

    if ((player->grounded && player->speedX == 0) || player->crouching)
    {
        SPR_setFrame(sprites[BOTTOM_SPRITE], 0);
    }
    else
    {
        SPR_setFrame(sprites[BOTTOM_SPRITE], playerObject->animationFrame);
    }
}

void PlayerCollide(ObjectStruct* playerObject, ObjectStruct* collidedObject)
{
}

void PlayerDestroy(ObjectStruct *playerObject)
{
}
