#include "Utils.h"
#include "LevelManager.h"
#include "Camera.h"
#include "Constants.h"

#include "Player.h"

u8 isRectOverlapping(fix32 left, fix32 top, fix32 right, fix32 bottom,
                     fix32 otherLeft, fix32 otherTop, fix32 otherRight, fix32 otherBottom)
{
    return !(left > otherRight || right < otherLeft || top > otherBottom || bottom < otherTop);
}

u8 isMeleeOverlapping(ObjectStruct *otherObject)
{
    return isRectOverlapping(
        otherObject->x + FIX32(otherObject->collisionRectLeft),
        otherObject->y + FIX32(otherObject->collisionRectTop),
        otherObject->x + FIX32(otherObject->collisionRectRight), 
        otherObject->y + FIX32(otherObject->collisionRectBottom),
        playerObject->x + FIX32(playerObject->attackRectLeft),
        playerObject->y + FIX32(playerObject->attackRectTop),
        playerObject->x + FIX32(playerObject->attackRectRight), 
        playerObject->y + FIX32(playerObject->attackRectBottom)
        );  
}

u8 isCollidingTile(u16 x, u16 y)
{
    //if (x >= currentMap->w || y >= currentMap->h || x < 0 || y < 0)
    if (x > currentMap->w || y > currentMap->h || x < 0 || y < 0)
    {
        return 0;
    }
    u16 tileIndex = x + currentMap->w * y;
    return (currentMap->tilemap[tileIndex] != baseTile && currentMap->tilemap[tileIndex] <= maxCollisionTile);
}

u8 isVisible(s16 startScreen, s16 endScreen, s16 x)
{
    return x >= startScreen && x <= endScreen;
}

u8 CheckBottomCollision(ObjectStruct *object)
{
    if (object->crouching)
    {
        ///POG
        return;
    }

    u16 xTile = fix32ToInt(object->x) >> 3;
    u16 yTile = fix32ToInt(object->y) >> 3;
    fix32 oldPosy = object->y;
    object->y += object->speedY;

    u8 overlap = 0;

    for (int i = 0; i < object->tileBottomCheckX; i++)
    {
        if (
            !isCollidingTile(xTile + i, yTile + object->tileBottomCheckY - 1) &&
            isCollidingTile(xTile + i, yTile + object->tileBottomCheckY) &&
            (object->speedY >= 0 || (oldPosy <= FIX32(yTile << 3))) &&
            isRectOverlapping(
                object->x + FIX32(object->rectLeft),
                object->y + FIX32(object->rectTop),
                object->x + FIX32(object->rectRight),
                object->y + FIX32(object->rectBottom),
                FIX32((xTile + i) << 3),
                FIX32((yTile + object->tileBottomCheckY) << 3),
                FIX32((xTile + i) << 3) + FIX32(8),
                FIX32((yTile + object->tileBottomCheckY) << 3) + FIX32(8)))
        {
            overlap = 1;
            break;
        }
    }

    if (!(object->y > FIX32(currentMap->h << 3)))
    {
        if (!overlap)
        {
            if (object->speedY < MAX_SPEED_Y)
            {
                object->speedY += GRAVITY;
            }
            object->grounded = 0;
        }
        else
        {
            object->y = FIX32(yTile << 3);
            object->grounded = 1;
            object->doubleJumping = 0;
            object->jumping = 0;
            object->speedY = 0;
        }
    }
    else
    {
        //dead
        object->speedY = 0;
    }
    return overlap;
}

u8 CheckTopCollision(ObjectStruct *object)
{
    u16 xTile = fix32ToInt(object->x) >> 3;
    u16 yTile = fix32ToInt(object->y) >> 3;
    u8 overlap = 0;
    if (object->jumping)
    {
        if (object->y > 0)
        {
            for (int i = 0; i < 3; i++)
            {
                if (isCollidingTile(xTile + i, yTile) &&
                    !isCollidingTile(xTile + i, yTile + 1) &&
                    isRectOverlapping(
                        object->x + FIX32(object->rectLeft),
                        object->y + FIX32(object->rectTop),
                        object->x + FIX32(object->rectRight),
                        object->y + FIX32(object->rectBottom),
                        FIX32((xTile + i) << 3),
                        FIX32((yTile + 1) << 3),
                        FIX32((xTile + i) << 3) + FIX32(8),
                        FIX32((yTile + 1) << 3) + FIX32(8)))
                {
                    overlap = 1;
                    break;
                }
            }
            if (overlap && object->speedY < 0)
            {
                object->speedY = FIX32(0);
                object->y = FIX32((yTile + 1) << 3);
            }
        }
    }
    return overlap;
}

u8 CheckSideCollision(ObjectStruct *object)
{
    u16 xTile = fix32ToInt(object->x) >> 3;
    u16 yTile = fix32ToInt(object->y) >> 3;
    u8 overlap = 0;
    s16 nextTile = 1;

    if (object->goingRight)
    {
        nextTile = object->tileSideCheckX;
    }

    if (object->simpleCollision)
    {
        if (isCollidingTile(xTile + nextTile, yTile))
        {
            overlap = 1;
        }
    }
    else
    {
        for (int i = 0; i < object->tileSideCheckY; i++)
        {
            if (
                (!isCollidingTile(xTile + nextTile + 1, yTile + i) || !isCollidingTile(xTile + nextTile - 1, yTile + i)) &&
                isCollidingTile(xTile + nextTile, yTile + i) &&
                isRectOverlapping(
                    object->x + FIX32(object->rectLeft),
                    object->y + FIX32(object->rectTop),
                    object->x + FIX32(object->rectRight),
                    object->y + FIX32(object->rectBottom - 1),
                    FIX32((xTile + nextTile) << 3),
                    FIX32((yTile + i) << 3),
                    FIX32((xTile + nextTile) << 3) + FIX32(8),
                    FIX32((yTile + i) << 3) + FIX32(8)))
            {
                overlap = 1;
                break;
            }
        }
    }
    if (overlap)
    {
        object->speedX = 0;
    }
    return overlap;
}

void Jump(ObjectStruct *object, fix32 speed)
{
    object->jumping++;
    object->grounded = 0;
    object->speedY = speed;
}

void DrawCollisionRect(ObjectStruct *object, s16 left, s16 top, s16 right, s16 bottom)
{
    SPR_setPosition(object->collisionSprite[0], fix32ToInt(object->x - camX) + left, fix32ToInt(object->y - camY) + top);
    SPR_setPosition(object->collisionSprite[1], fix32ToInt(object->x - camX) + right - 8, fix32ToInt(object->y - camY) + top);
    SPR_setPosition(object->collisionSprite[2], fix32ToInt(object->x - camX) + left, fix32ToInt(object->y - camY) + bottom - 8);
    SPR_setPosition(object->collisionSprite[3], fix32ToInt(object->x - camX) + right - 8, fix32ToInt(object->y - camY) + bottom - 8);
}

void CollisionFunctionStub(ObjectStruct* objectStruct, ObjectStruct* collidedObject)
{
}